@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005, Magnus Hjorth
+ * Copyright (C) 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -21,6 +21,7 @@
 #include <config.h>
 
 
+#include "um.h"
 #include "document.h"
 #include "filetypes.h"
 #include "player.h"
@@ -89,6 +90,7 @@ static void document_init(GtkObject *obj)
 {
      Document *d = DOCUMENT(obj);
      d->filename = NULL;
+     d->lossy = FALSE;
      d->titlename = NULL;
      d->title_serial = 0;
      d->history_pos = NULL;
@@ -198,9 +200,13 @@ GtkType document_get_type(void)
 Document *document_new_with_file(gchar *filename, StatusBar *bar)
 {
      Chunk *c;
-     c = chunk_load(filename,dither_editing,bar);
+     gboolean b;
+     Document *d;
+     c = chunk_load_x(filename,dither_editing,bar,&b);
      if (c == NULL) return NULL;
-     return document_new_with_chunk(c,filename,bar);
+     d = document_new_with_chunk(c,filename,bar);
+     d->lossy = b;
+     return d;
 }
 
 static void document_set_filename(Document *d, gchar *filename, 
@@ -209,7 +215,7 @@ static void document_set_filename(Document *d, gchar *filename,
      guint i=0;
      GList *l;
      Document *x;
-     if (filename == d->filename || 
+     if ((filename == d->filename && filename != NULL) || 
 	 (d->filename != NULL && filename != NULL && 
 	  !strcmp(d->filename,filename)))
 	  return;
@@ -222,8 +228,8 @@ static void document_set_filename(Document *d, gchar *filename,
 	  for (l=document_objects->list;l!=NULL;l=l->next) {
 	       x = DOCUMENT(l->data);
 	       if (x == d) continue;
-	       if (d->filename == NULL || strcmp(namepart(d->filename),
-						 namepart(x->filename)))
+	       if (d->filename == NULL || x->filename == NULL ||
+		   strcmp(namepart(d->filename),namepart(x->filename)))
 		    continue;
 	       if (x->title_serial > i) i=x->title_serial;
 	  }
@@ -256,9 +262,20 @@ Document *document_new_with_chunk(Chunk *chunk, gchar *sourcename,
      return d;
 }
 
-gboolean document_save(Document *d, gchar *filename)
+gboolean document_save(Document *d, gchar *filename, gint type_id, 
+		       gboolean use_defs)
 {
-     gboolean b = chunk_save(d->chunk,filename,dither_editing,d->bar);
+     gboolean b;
+     int i;
+     if (d->filename != NULL && !strcmp(filename,d->filename) &&
+	 d->lossy) {
+	  i = user_message(_("Loading and then saving files with 'lossy' "
+			     "formats (like mp3 and ogg) leads to a quality "
+			     "loss, also for the unmodified parts of the "
+			     "file."), UM_OKCANCEL);
+	  if (i == MR_CANCEL) return TRUE;
+     }
+     b = chunk_save(d->chunk,filename,type_id,use_defs,dither_editing,d->bar);
      if (!b) {
 	  clear_history(d->history_pos);
 	  d->history_pos = NULL;
@@ -633,6 +650,7 @@ void document_set_view(Document *d, off_t viewstart, off_t viewend)
 {
      off_t o;
 
+     /* puts("document_set_view"); */
      if (viewstart > viewend) {
 	  o = viewstart;
 	  viewstart = viewend;
@@ -684,9 +702,10 @@ void document_set_selection(Document *d, off_t selstart, off_t selend)
 }
 
 void document_zoom(Document *d, gfloat zoom, gboolean followcursor)
-{
+{     
      off_t dist,newdist;
      off_t newstart,newend;
+     /* puts("document_zoom"); */
      dist = d->viewend - d->viewstart;
      newdist = (off_t) ((gfloat) dist / zoom);
      if (newdist >= d->chunk->length) {
@@ -694,16 +713,19 @@ void document_zoom(Document *d, gfloat zoom, gboolean followcursor)
 	  return;
      }
      if (newdist < 1) newdist = 1;
+     /* printf("dist: %d, newdist: %d\n",(int)dist,(int)newdist);
+	printf("d->cursorpos: %d, d->viewstart: %d, d->viewend: %d\n",
+	(int)d->cursorpos,(int)d->viewstart,(int)d->viewend); */
      if (followcursor && d->cursorpos >= d->viewstart && 
 	 d->cursorpos < d->viewend)
-	  newstart = d->cursorpos - dist/2;
+	  newstart = d->cursorpos - newdist/2;
      else
 	  newstart = d->viewstart + (dist - newdist)/2;
      if (newstart < 0) newstart = 0;
      newend = newstart + newdist;
      if (newend > d->chunk->length) {
 	  newend = d->chunk->length;
-	  newstart = newend - dist;
+	  newstart = newend - newdist;
      }
      document_set_view(d, newstart, newend);
 }
