@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -34,6 +34,7 @@
 #include "main.h"
 #include "filetypes.h"
 #include "tempfile.h"
+#include "combo.h"
 #include "gettext.h"
 
 #define MAX_REAL_SIZE inifile_get_guint32(INI_SETTING_REALMAX,INI_SETTING_REALMAX_DEFAULT)
@@ -45,11 +46,14 @@ struct file_type;
 struct file_type {
      gchar *name;
      gchar *extension;
+     gboolean lossy;
      gboolean (*typecheck)(gchar *filename);
      Chunk *(*load)(gchar *filename, int dither_mode, StatusBar *bar);
-     gboolean (*save)(Chunk *chunk, gchar *filename, 
+     gboolean (*save)(Chunk *chunk, gchar *filename, gpointer settings,
 		      struct file_type *type, int dither_mode, StatusBar *bar,
 		      gboolean *fatal);
+     gpointer (*get_settings)(void);
+     void (*free_settings)(gpointer settings);
      gint extra_data;
 };
 
@@ -57,14 +61,14 @@ static GList *file_types = NULL;
 
 static gboolean wav_check(gchar *filename);
 static Chunk *wav_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean wav_save(Chunk *chunk, gchar *filename, 
+static gboolean wav_save(Chunk *chunk, gchar *filename, gpointer settings,
 			 struct file_type *type, int dither_mode, 
 			 StatusBar *bar,gboolean *fatal);
 
 #ifdef HAVE_LIBSNDFILE
 static gboolean sndfile_check(gchar *filename);
 static Chunk *sndfile_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean sndfile_save(Chunk *chunk, gchar *filename, 
+static gboolean sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
 			     struct file_type *type, int dither_mode, 
 			     StatusBar *bar, gboolean *fatal);
 static gboolean sndfile_save_main(Chunk *chunk, gchar *filename, 
@@ -73,17 +77,20 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
 #endif
 
 static Chunk *raw_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean raw_save(Chunk *chunk, gchar *filename, 
+static gboolean raw_save(Chunk *chunk, gchar *filename, gpointer settings,
 			 struct file_type *type, int dither_mode,
 			 StatusBar *bar, gboolean *fatal);
 
 static Chunk *ogg_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean ogg_save(Chunk *chunk, gchar *filename, struct file_type *type,
+static gboolean ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
+			 struct file_type *type,
 			 int dither_mode, StatusBar *bar, gboolean *fatal);
 
 
+static gpointer mp3_get_settings(void);
 static Chunk *mp3_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean mp3_save(Chunk *chunk, gchar *filename, struct file_type *type,
+static gboolean mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
+			 struct file_type *type,
 			 int dither_mode, StatusBar *bar, gboolean *fatal);
 
 static Chunk *try_mplayer(gchar *filename, int dither_mode, StatusBar *bar);
@@ -106,37 +113,40 @@ static gboolean xunsetenv(char *varname)
      return FALSE;
 }
 
-static void register_file_type(gchar *name, gchar *ext, 
-			       gboolean (*typecheck)(gchar *filename),
-			       Chunk *(*load)(gchar *filename, 
-					      int dither_mode, StatusBar *bar),
-			       gboolean (*save)(Chunk *chunk, gchar *filename, 
-						struct file_type *type,
-						int dither_mode,
-						StatusBar *bar,
-						gboolean *fatal),
-			       int extra_data)
+static struct file_type *register_file_type
+(gchar *name, gchar *ext, gboolean lossy,
+ gboolean (*typecheck)(gchar *filename), 
+ Chunk *(*load)(gchar *filename, int dither_mode, StatusBar *bar),
+ gboolean (*save)(Chunk *chunk, gchar *filename,gpointer settings,
+		  struct file_type *type,int dither_mode,
+		  StatusBar *bar,gboolean *fatal),
+ int extra_data)
 {
      struct file_type *t;
      t = g_malloc(sizeof(*t));
      t->name = g_strdup(name);
      t->extension = g_strdup(ext);
+     t->lossy = lossy;
      t->typecheck = typecheck;
      t->load = load;
      t->save = save;
      t->extra_data = extra_data;
+     t->get_settings = NULL;
+     t->free_settings = NULL;
      file_types = g_list_append(file_types,t);
+     return t;
 }
 
 static void setup_types(void)
 {
+     struct file_type *t;
 #if defined(HAVE_LIBSNDFILE)
      SF_FORMAT_INFO info;
      int major_count,i;
      gchar buf[32];
 #endif
      if (file_types) return;
-     register_file_type(_("Microsoft WAV format"), ".wav", wav_check, 
+     register_file_type(_("Microsoft WAV format"), ".wav", FALSE, wav_check, 
 			wav_load, wav_save, 0);
 #if defined(HAVE_LIBSNDFILE)
      sf_command(NULL, SFC_GET_FORMAT_MAJOR_COUNT, &major_count, sizeof(int));
@@ -149,17 +159,20 @@ static void setup_types(void)
 	  if ((info.format&SF_FORMAT_TYPEMASK) == SF_FORMAT_RAW ||
 	      (info.format&SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV) continue;
 	  snprintf(buf,sizeof(buf),".%s",info.extension);
-	  register_file_type((gchar *)info.name, buf, sndfile_check, 
+	  register_file_type((gchar *)info.name, buf, FALSE, sndfile_check, 
 			     sndfile_load, sndfile_save, 
 			     info.format&SF_FORMAT_TYPEMASK);
      }
 #endif
      if (program_exists("oggenc") || program_exists("oggdec"))
-	  register_file_type(_("Ogg Vorbis"),".ogg", NULL, ogg_load, 
+	  register_file_type(_("Ogg Vorbis"),".ogg", TRUE, NULL, ogg_load, 
 			     ogg_save, 0);
-     if (program_exists("lame"))
-	    register_file_type("MP3",".mp3",NULL,mp3_load,mp3_save,0);
-     register_file_type(_("Raw PCM data"), ".raw", NULL, raw_load, 
+     if (program_exists("lame")) {
+	    t = register_file_type("MP3",".mp3",TRUE,NULL,mp3_load,mp3_save,0);
+	    t->get_settings = mp3_get_settings;
+	    t->free_settings = g_free;
+     }
+     register_file_type(_("Raw PCM data"), ".raw", FALSE,NULL, raw_load, 
 			raw_save, 0);
 }
 
@@ -185,7 +198,16 @@ gchar *fileformat_extension(guint fileformat)
      return NULL;
 }
 
-static Chunk *chunk_load_main(gchar *filename, int dither_mode, StatusBar *bar)
+gboolean fileformat_has_options(guint fileformat)
+{
+     struct file_type *t;
+     t = g_list_nth_data(file_types, fileformat);
+     if (t) return ((t->get_settings) != NULL);
+     else return FALSE;
+}
+
+static Chunk *chunk_load_main(gchar *filename, int dither_mode, StatusBar *bar,
+			      struct file_type **format)
 {
      GList *l;
      struct file_type *ft;
@@ -200,15 +222,19 @@ static Chunk *chunk_load_main(gchar *filename, int dither_mode, StatusBar *bar)
 	  if (!ft->extension) continue;
 	  c = strchr(filename,0) - strlen(ft->extension);
 	  if (c<filename || strcmp(c,ft->extension)!=0) continue;
-	  if (ft->typecheck==NULL || ft->typecheck(filename)) 
+	  if (ft->typecheck==NULL || ft->typecheck(filename)) {
+	       *format = ft;
 	       return ft->load(filename,dither_mode,bar);
+	  }
      }
      /* Use the file checking functions */
      for (l=file_types; l!=NULL; l=l->next) {
 	  ft = (struct file_type *)l->data;
 	  if (ft->typecheck == NULL) continue;
-	  if (ft->typecheck(filename))
+	  if (ft->typecheck(filename)) {
+	       *format = ft;
 	       return ft->load(filename,dither_mode,bar);
+	  }
      }
      /* Try mplayer if available */
      chunk = try_mplayer(filename,dither_mode,bar);
@@ -217,11 +243,13 @@ static Chunk *chunk_load_main(gchar *filename, int dither_mode, StatusBar *bar)
      return raw_load(filename,dither_mode,bar);
 }
 
-Chunk *chunk_load(gchar *filename, int dither_mode, StatusBar *bar )
+Chunk *chunk_load_x(gchar *filename, int dither_mode, StatusBar *bar,
+		    gboolean *lossy)
 {
      Chunk *chunk;
      gchar *c;
      EFILE *f;
+     struct file_type *ft;
 
      /* First, see if the file exists */
      if (!file_exists(filename)) {
@@ -245,12 +273,19 @@ Chunk *chunk_load(gchar *filename, int dither_mode, StatusBar *bar )
      e_fclose(f);
 
      status_bar_begin_progress(bar,1,_("Loading"));
-     chunk = chunk_load_main(filename,dither_mode,bar);
+     chunk = chunk_load_main(filename,dither_mode,bar,&ft);
      status_bar_end_progress(bar);
 
+     if (chunk != NULL) *lossy = ft->lossy;
      return chunk;
 }
 
+Chunk *chunk_load(gchar *filename, int dither_mode, StatusBar *bar)
+{
+     gboolean b;
+     return chunk_load_x(filename,dither_mode,bar,&b);
+}
+
 
 static struct file_type *choose_format(gchar *filename)
 {
@@ -290,19 +325,37 @@ static struct file_type *choose_format(gchar *filename)
      return ft;
 }
 
-gboolean chunk_save(Chunk *chunk, gchar *filename, int dither_mode, 
+gboolean chunk_save(Chunk *chunk, gchar *filename, int filetype, 
+		    gboolean use_default_settings, int dither_mode, 
 		    StatusBar *bar)
 {
      gchar *d;
      gboolean b=FALSE,res;
      EFILE *f;
+     gpointer settings = NULL;
      struct file_type *ft;
+     gint i;
      
      /* Let the user choose a format first, so that if the user presses cancel,
       * we haven't done any of the backup/unlink stuff.. */	
-     ft = choose_format(filename);
+     if (filetype < 0) 
+	  ft = choose_format(filename);
+     else
+	  ft = g_list_nth_data(file_types, filetype);
      if (ft == NULL) return TRUE;
 
+     if (!use_default_settings) {
+	  if (ft->get_settings == NULL) {
+	       i = user_message("There are no settings for this file format. "
+				"Proceeding with standard settings.",
+				UM_OKCANCEL);
+	       if (i == MR_CANCEL) return TRUE;
+	  } else {
+	       settings = ft->get_settings();
+	       if (settings == NULL) return TRUE;
+	  }
+     }
+     
      if (file_exists(filename)) {
 
           /* Check that the file is writeable by us. */
@@ -315,7 +368,7 @@ gboolean chunk_save(Chunk *chunk, gchar *filename, int dither_mode,
      }
 
      status_bar_begin_progress(bar,chunk->size,_("Saving"));
-     res = ft->save(chunk,filename,ft,dither_mode,bar,&b);
+     res = ft->save(chunk,filename,settings,ft,dither_mode,bar,&b);
 
      if (res && b && !status_bar_progress(bar,0)) {
 	  d = g_strdup_printf(_("The file %s may be destroyed since the"
@@ -328,21 +381,11 @@ gboolean chunk_save(Chunk *chunk, gchar *filename, int dither_mode,
      }
      status_bar_end_progress(bar);
 
+     if (settings != NULL) ft->free_settings(settings);
+
      return res;
 }
 
-gboolean chunk_save_ff(Chunk *chunk, gchar *filename,
-		       guint fileformat, int dither_mode, StatusBar *bar)
-{
-     struct file_type *ft;
-     gboolean b;
-     setup_types();
-     ft = g_list_nth_data( file_types, fileformat );
-     g_assert (ft != NULL);
-     return ft->save(chunk, filename, ft, dither_mode, bar, &b);
-}
-
-
 /* WAV */
 
 
@@ -475,7 +518,7 @@ static Chunk *wav_load(char *filename, int dither_mode, StatusBar *bar)
      return chunk_new_from_datasource(ds);
 }
 
-static gboolean wav_save(Chunk *chunk, char *filename, 
+static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
 			 struct file_type *type, int dither_mode, 
 			 StatusBar *bar, gboolean *fatal)
 {
@@ -586,7 +629,7 @@ static Chunk *raw_load(gchar *filename, int dither_mode, StatusBar *bar)
      return chunk_new_from_datasource(ds);
 }
 
-static gboolean raw_save(Chunk *chunk, gchar *filename, 
+static gboolean raw_save(Chunk *chunk, gchar *filename, gpointer settings,
 			 struct file_type *type, int dither_mode, 
 			 StatusBar *bar, gboolean *fatal)
 {
@@ -815,7 +858,7 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
 
 }
 
-static gboolean sndfile_save(Chunk *chunk, gchar *filename, 
+static gboolean sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
 			     struct file_type *type, int dither_mode, 
 			     StatusBar *bar, gboolean *fatal)
 {
@@ -900,7 +943,8 @@ static Chunk *ogg_load(gchar *filename, int dither_mode, StatusBar *bar)
      return c;
 }
 
-static gboolean ogg_save(Chunk *chunk, gchar *filename, struct file_type *type,
+static gboolean ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
+			 struct file_type *type,
 			 int dither_mode, StatusBar *bar, gboolean *fatal)
 {
      Chunk *x=NULL,*y;
@@ -935,7 +979,214 @@ static gboolean ogg_save(Chunk *chunk, gchar *filename, struct file_type *type,
      return FALSE;
 }
 
+static struct {
+     Combo *type_combo, *subtype_combo;
+     GtkEntry *arg_entry;
+     GtkToggleButton *set_default;
+     gboolean ok_flag, destroyed_flag;
+     GList *preset_list, *bitrate_list;
+} mp3_get_settings_data;
 
+static void set_flag(gboolean *flag) {
+     *flag = TRUE;
+}
+
+static void mp3_get_settings_type_changed(Combo *obj, gpointer user_data)
+{
+     int i;
+     i = combo_selected_index(obj);
+     gtk_widget_set_sensitive(GTK_WIDGET(mp3_get_settings_data.subtype_combo),
+			      (i<3));
+     gtk_widget_set_sensitive(GTK_WIDGET(mp3_get_settings_data.arg_entry),
+			      (i==3));
+     if (i == 0)
+	  combo_set_items(mp3_get_settings_data.subtype_combo, 
+			  mp3_get_settings_data.preset_list, 0);
+     else if (i < 3)
+	  combo_set_items(mp3_get_settings_data.subtype_combo,
+			  mp3_get_settings_data.bitrate_list, 5);
+     
+}
+
+static gpointer mp3_get_settings(void)
+{
+     gchar *type_strings[] = { "vbr_preset", "abr_preset", "cbr_preset", 
+			       "custom" };
+     gchar *type_names[] = { _("Variable bitrate (default)"),
+			     _("Average bitrate"),
+			     _("Constant bitrate"),
+			     _("Custom argument") };
+     gchar *bitrate_strings[] = { "80", "96", "112", "128", "160", "192", 
+				  "224", "256", "320" };
+     gchar *vbr_preset_strings[] = { "standard", "extreme", "insane" };
+     gchar *vbr_preset_names[] = { _("Standard (high quality)"), 
+				   _("Extreme (higher quality)"),
+				   _("Insane (highest possible quality)") };
+     
+     
+     int type; /* 0 = VBR preset, 1 = ABR preset, 2 = CBR preset, 3 = custom */
+     int subtype = 0;
+     gchar *custom_arg = NULL;
+
+     GList *l; 
+     GtkWidget *a,*b,*c,*d;
+
+     gchar *p;
+     int i,j;
+
+     /* Setup lists */
+     for (l=NULL, i=0; i<ARRAY_LENGTH(vbr_preset_names); i++)
+	  l = g_list_append(l, vbr_preset_names[i]);          
+     mp3_get_settings_data.preset_list = l;
+     for (l=NULL, i=0; i<ARRAY_LENGTH(bitrate_strings); i++) {
+	  p = g_strdup_printf(_("%s kbit/s"),bitrate_strings[i]);
+	  l = g_list_append(l,p);
+     }
+     mp3_get_settings_data.bitrate_list = l;
+
+     /* Get default settings */     
+     p = inifile_get("mp3_EncodingType",type_strings[0]);
+     for (i=0; i<ARRAY_LENGTH(type_strings); i++)
+	  if (!strcmp(p,type_strings[i])) break;    
+     if (i < ARRAY_LENGTH(type_strings)) type = i;
+     else type = 0;
+     p = inifile_get("mp3_EncodingArg","");
+     if (type == 0) {
+	  for (i=0; i<ARRAY_LENGTH(vbr_preset_strings); i++) 
+	       if (!strcmp(p,vbr_preset_strings[i])) break;
+	  if (i < ARRAY_LENGTH(vbr_preset_strings)) subtype = i;
+	  else subtype = 0;
+     } else if (type < 3) {
+	  for (i=0; i<ARRAY_LENGTH(bitrate_strings); i++)
+	       if (!strcmp(p,bitrate_strings[i])) break;
+	  if (i<ARRAY_LENGTH(bitrate_strings)) subtype = i;
+	  else subtype = 5;
+     } else {
+	  custom_arg = p;
+     }
+
+     /* Create the window */
+
+     a = gtk_window_new(GTK_WINDOW_DIALOG);
+     gtk_window_set_title(GTK_WINDOW(a),_("MP3 Preferences"));
+     gtk_window_set_modal(GTK_WINDOW(a),TRUE);
+     gtk_signal_connect_object(GTK_OBJECT(a),"destroy",
+			       GTK_SIGNAL_FUNC(set_flag),
+			       (GtkObject *)
+			       &(mp3_get_settings_data.destroyed_flag));
+     b = gtk_vbox_new(FALSE,6);
+     gtk_container_add(GTK_CONTAINER(a),b);
+     gtk_container_set_border_width(GTK_CONTAINER(b),6);
+     c = gtk_hbox_new(FALSE,6);
+     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
+     d = gtk_label_new(_("Encoding type: "));
+     gtk_box_pack_start(GTK_BOX(c),d,FALSE,FALSE,0);
+     for (l=NULL, i=0; i<ARRAY_LENGTH(type_names); i++)
+	  l = g_list_append(l,type_names[i]);     
+     d = combo_new();
+     mp3_get_settings_data.type_combo = COMBO(d);
+     combo_set_items(mp3_get_settings_data.type_combo, l, 0);
+     gtk_signal_connect(GTK_OBJECT(d),"selection_changed",
+			GTK_SIGNAL_FUNC(mp3_get_settings_type_changed),NULL);
+     gtk_box_pack_start(GTK_BOX(c),d,TRUE,TRUE,0);     
+     g_list_free(l);
+     c = gtk_hbox_new(FALSE,4);
+     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
+     d = gtk_label_new(_("Quality: "));
+     gtk_box_pack_start(GTK_BOX(c),d,FALSE,FALSE,0);
+     d = combo_new();
+     mp3_get_settings_data.subtype_combo = COMBO(d);
+     combo_set_items(mp3_get_settings_data.subtype_combo, 
+		     mp3_get_settings_data.preset_list, 0);
+     gtk_box_pack_start(GTK_BOX(c),d,TRUE,TRUE,0);
+     c = gtk_hbox_new(FALSE, 4);
+     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
+     d = gtk_label_new(_("Custom argument: "));
+     gtk_box_pack_start(GTK_BOX(c),d,FALSE,FALSE,0);
+     d = gtk_entry_new();
+     mp3_get_settings_data.arg_entry = GTK_ENTRY(d);
+     gtk_widget_set_sensitive(d,FALSE);
+     gtk_box_pack_start(GTK_BOX(c),d,TRUE,TRUE,0);
+     c = gtk_check_button_new_with_label(_("Use this setting by default"));
+     mp3_get_settings_data.set_default = GTK_TOGGLE_BUTTON(c);
+     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
+     c = gtk_hbutton_box_new();
+     gtk_box_pack_end(GTK_BOX(b),c,FALSE,FALSE,0);
+     d = gtk_button_new_with_label(_("OK"));
+     gtk_signal_connect_object(GTK_OBJECT(d),"clicked",
+			       GTK_SIGNAL_FUNC(set_flag),
+			       (GtkObject *)&(mp3_get_settings_data.ok_flag));
+     gtk_signal_connect_object(GTK_OBJECT(d),"clicked",
+			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
+			       GTK_OBJECT(a));
+     gtk_container_add(GTK_CONTAINER(c),d);
+     d = gtk_button_new_with_label(_("Cancel"));
+     gtk_signal_connect_object(GTK_OBJECT(d),"clicked",
+			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
+			       GTK_OBJECT(a));
+     gtk_container_add(GTK_CONTAINER(c),d);
+     c = gtk_hseparator_new();
+     gtk_box_pack_end(GTK_BOX(b),c,FALSE,FALSE,0);
+     gtk_widget_show_all(a);
+     
+     gtk_object_ref(GTK_OBJECT(mp3_get_settings_data.type_combo));
+     gtk_object_ref(GTK_OBJECT(mp3_get_settings_data.subtype_combo));
+     gtk_object_ref(GTK_OBJECT(mp3_get_settings_data.arg_entry));
+     gtk_object_ref(GTK_OBJECT(mp3_get_settings_data.set_default));
+
+     mp3_get_settings_data.destroyed_flag = FALSE;
+     mp3_get_settings_data.ok_flag = FALSE;
+
+     combo_set_selection(mp3_get_settings_data.type_combo, type);
+     if (type < 3)
+	  combo_set_selection(mp3_get_settings_data.subtype_combo, subtype);
+     else
+	  gtk_entry_set_text(mp3_get_settings_data.arg_entry, custom_arg);
+
+     while (!mp3_get_settings_data.destroyed_flag)
+	  mainloop(TRUE);
+
+     if (mp3_get_settings_data.ok_flag) {
+	  i = combo_selected_index(mp3_get_settings_data.type_combo);
+	  j = combo_selected_index(mp3_get_settings_data.subtype_combo);
+	  switch (i) {
+	  case 0: 
+	       p = g_strdup_printf("--preset %s",vbr_preset_strings[j]); 
+	       break;
+	  case 1: 
+	       p = g_strdup_printf("--preset %s",bitrate_strings[j]);
+	       break;
+	  case 2:
+	       p = g_strdup_printf("--preset cbr %s",bitrate_strings[j]);
+	       break;
+	  case 3:
+	       p=g_strdup(gtk_entry_get_text(mp3_get_settings_data.arg_entry));
+	       break;
+	  default:
+	       g_assert_not_reached();
+	  }
+	  if (gtk_toggle_button_get_active(mp3_get_settings_data.set_default)){
+	       inifile_set("mp3_EncodingType",type_strings[i]);
+	       if (i == 0)
+		    inifile_set("mp3_EncodingArg",vbr_preset_strings[j]);
+	       else if (i < 3)
+		    inifile_set("mp3_EncodingArg",bitrate_strings[j]);
+	       else
+		    inifile_set("mp3_EncodingArg",p);
+	  }	  
+     } else p = NULL;
+
+     gtk_object_unref(GTK_OBJECT(mp3_get_settings_data.type_combo));
+     gtk_object_unref(GTK_OBJECT(mp3_get_settings_data.subtype_combo));
+     gtk_object_unref(GTK_OBJECT(mp3_get_settings_data.arg_entry));
+     gtk_object_unref(GTK_OBJECT(mp3_get_settings_data.set_default));
+     
+     g_list_free(mp3_get_settings_data.preset_list);
+     g_list_foreach(mp3_get_settings_data.bitrate_list,(GFunc)g_free,NULL);
+     g_list_free(mp3_get_settings_data.bitrate_list);
+
+     return p;
+}
 
 static Chunk *mp3_load(gchar *filename, int dither_mode, StatusBar *bar)
 {
@@ -953,7 +1204,8 @@ static gboolean wav_regular_format(Dataformat *fmt)
      return (XOR(fmt->samplesize == 1, fmt->sign));
 }
 
-static gboolean mp3_save(Chunk *chunk, gchar *filename, struct file_type *type,
+static gboolean mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
+			 struct file_type *type,
 			 int dither_mode, StatusBar *bar, gboolean *fatal)
 {
      Chunk *x;
@@ -963,9 +1215,12 @@ static gboolean mp3_save(Chunk *chunk, gchar *filename, struct file_type *type,
      xunlink(filename);
      c = g_strdup_printf("OUTFILE=%s",filename);
      if (xputenv(c)) { g_free(c); return TRUE; }
+     c = g_strdup_printf("LAMEFLAGS=%s",(settings == NULL) ? 
+			 "--preset standard" : (gchar *)settings);
+     if (xputenv(c)) { g_free(c); return TRUE; }
      if (wav_regular_format(&(chunk->format)))
 	 b = pipe_dialog_send_chunk(chunk,
-				    "lame --silent --preset standard - "
+				    "lame --silent $LAMEFLAGS - "
 				    "\"$OUTFILE\"",TRUE,dither_mode,bar);
      else {
 	  memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
