@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -231,11 +231,34 @@ static void update_desc(Mainwindow *w)
 			      
 }
 
+static void typesel_changed(Combo *obj, GtkWidget *user_data)
+{
+     int i;
+     gchar *c;
+     i = combo_selected_index(obj);
+     if (i == 0) {
+	  gtk_widget_set_sensitive(user_data,TRUE);
+     } else {
+	  c = fileformat_extension(i-1);
+	  get_filename_modify_extension(c);
+	  if (fileformat_has_options(i-1)) {
+	       gtk_widget_set_sensitive(user_data,TRUE);
+	  } else {
+	       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data),TRUE);
+	       gtk_widget_set_sensitive(user_data,FALSE);
+	  }
+     }
+}
 
-static gchar *get_save_filename(gchar *old_filename, gchar *title_text)
+static gchar *get_save_filename(gchar *old_filename, gchar *title_text, 
+				gint *type_id, gboolean *use_defaults)
 {
      gchar *lsf,*lsd,*filename;
-     gchar *c,*d;
+     gchar *c,*d,*e;
+     GtkBox *b1,*b2;
+     GtkWidget *w,*usedef,*typesel;
+     GList *l;
+     gint i;
 
      /* Get the lastSaveFile entry */
      lsf = inifile_get("lastSaveFile",NULL);
@@ -252,30 +275,73 @@ static gchar *get_save_filename(gchar *old_filename, gchar *title_text)
 	       lsd = NULL;
 	  }
      } else lsd = NULL;
+
+     /* Create the custom widget */
+     usedef = gtk_check_button_new_with_label(_("Use default settings"));
+     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(usedef),TRUE);
+     typesel = combo_new();
      
+     l = g_list_append(NULL,g_strdup(_("Auto-detect from extension")));
+     for (i=0; ; i++) {
+	  c = fileformat_name(i);
+	  if (c == NULL) break;
+	  d = fileformat_extension(i);
+	  e = g_strdup_printf("%s (%s)",c,d);
+	  l = g_list_append(l,e);
+     }
+     combo_set_items(COMBO(typesel),l,0);
+     gtk_signal_connect(GTK_OBJECT(typesel),"selection-changed",
+			GTK_SIGNAL_FUNC(typesel_changed),usedef);
+     g_list_foreach(l,(GFunc)g_free,NULL);
+     g_list_free(l);
+
+     b1 = GTK_BOX(gtk_hbox_new(FALSE,0));
+     w = gtk_label_new(_("File type: "));
+     gtk_box_pack_start(b1,w,FALSE,FALSE,0);
+     gtk_box_pack_start(b1,typesel,TRUE,TRUE,0);
+     gtk_box_pack_start(b1,usedef,FALSE,FALSE,6);
+     b2 = GTK_BOX(gtk_vbox_new(FALSE,8));
+     gtk_box_pack_start(b2,GTK_WIDGET(b1),FALSE,FALSE,0);
+
+     gtk_object_ref(GTK_OBJECT(usedef));
+     gtk_object_ref(GTK_OBJECT(typesel));
+
+     gtk_widget_show_all(GTK_WIDGET(b2));
+
      if (lsd == NULL)
-	  filename = get_filename ( old_filename, "*.wav", title_text, TRUE );
+	  filename = get_filename ( old_filename, "*.wav", title_text, 
+				    TRUE, GTK_WIDGET(b2));
      else if (old_filename == NULL)
-	  filename = get_filename ( lsd, "*.wav", title_text, TRUE );
+	  filename = get_filename (lsd, "*.wav", title_text, TRUE, 
+				   GTK_WIDGET(b2));
      else { 
 	  c = strrchr(old_filename, '/');
 	  if (!c) c=old_filename;
 	  else c = c+1;
 	  d = g_strjoin("/",lsd,c,NULL);
-	  filename = get_filename( d, "*.wav", title_text, TRUE);
+	  filename = get_filename(d, "*.wav", title_text, TRUE, 
+				  GTK_WIDGET(b2));
 	  g_free(d);
      }
+
+     *use_defaults = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(usedef));
+     *type_id = combo_selected_index(COMBO(typesel)) - 1;
+
      g_free(lsd);
+     gtk_object_unref(GTK_OBJECT(typesel));
+     gtk_object_unref(GTK_OBJECT(usedef));
      return filename;
      
 }
 
 static gboolean mainwindow_save ( Mainwindow *w, gchar *filename )
 {
+     int type_id = -1;
+     gboolean use_defs = TRUE;
      gboolean r;
      gboolean fmal = FALSE; /* TRUE if filename should be g_free:d */    
 
-     if (!document_can_undo(w->doc))
+     if (!document_can_undo(w->doc) && w->doc->filename != NULL)
 	  if (user_message(_("The file has not changed since last save. Press OK"
 			   " if you want to save it anyway?"),UM_OKCANCEL) == 
 	      MR_CANCEL) 
@@ -285,14 +351,15 @@ static gboolean mainwindow_save ( Mainwindow *w, gchar *filename )
       * box. Also set the fmal flag so we remember to free the
       * filename later. */  
      if (filename == NULL) {
-	  filename = get_save_filename(w->doc->filename,_("Save File"));
+	  filename = get_save_filename(w->doc->filename,_("Save File"),
+				       &type_id,&use_defs);
 	  if (filename == NULL) return TRUE;
 	  fmal = TRUE;
      }
 
      g_assert(filename != NULL);
      /* Save the file */
-     r = document_save(w->doc, filename);
+     r = document_save(w->doc, filename, type_id, use_defs);
      if (r) { if (fmal) g_free(filename); return TRUE; }
 
      inifile_set("lastSaveFile",filename);
@@ -519,14 +586,13 @@ static void mainwindow_class_init(GtkObjectClass *klass)
 
 }
 
-static void mainwindow_set_chunk(Mainwindow *w, Chunk *c, gchar *filename)
+static void mainwindow_set_document(Mainwindow *w, Document *d, 
+				    gchar *filename)
 {
-     Document *d;
      if (w->doc != NULL) {
 	  w = MAINWINDOW ( mainwindow_new() );
 	  gtk_widget_show ( GTK_WIDGET ( w ) );
      }
-     d = document_new_with_chunk(c,filename,w->statusbar);
      w->doc = d;
      document_set_followmode(d,w->followmode);
      gtk_object_ref(GTK_OBJECT(w->doc));
@@ -549,6 +615,13 @@ static void mainwindow_set_chunk(Mainwindow *w, Chunk *c, gchar *filename)
      mainwindow_view_changed(w->doc,w);
 }
 
+static void mainwindow_set_chunk(Mainwindow *w, Chunk *c, gchar *filename)
+{
+     Document *d;
+     d = document_new_with_chunk(c,filename,w->statusbar);
+     mainwindow_set_document(w,d,filename);
+}
+
 void mainwindow_set_speed_sensitive(gboolean sensitive)
 {
      GList *l;
@@ -562,22 +635,22 @@ void mainwindow_set_speed_sensitive(gboolean sensitive)
 static void file_open(GtkMenuItem *menuitem, gpointer user_data)
 {
      gchar *c;
-     Chunk *chunk;
+     Document *doc;
      Mainwindow *w = MAINWINDOW ( user_data );     
      if (w->doc != NULL && w->doc->filename != NULL)
-	  c = get_filename(w->doc->filename,"*.wav", _("Load File"), FALSE );
+	  c = get_filename(w->doc->filename,"*.wav",_("Load File"),FALSE,NULL);
      else {
 	  c = inifile_get("lastOpenFile",NULL);
 	  if (c == NULL) c = inifile_get("lastSaveFile",NULL);
-	  c = get_filename(c,"*.wav", _("Load File"), FALSE);
+	  c = get_filename(c,"*.wav", _("Load File"), FALSE, NULL);
      }
      if (!c) return;
-     chunk = chunk_load ( c, dither_editing, w->statusbar );
-     if (!chunk) {
+     doc = document_new_with_file ( c, w->statusbar );
+     if (doc == NULL) {
 	  g_free(c);
 	  return;
      }
-     mainwindow_set_chunk ( w, chunk, c );
+     mainwindow_set_document ( w, doc, c );
      inifile_set("lastOpenFile",c);
      recent_file(c);
      g_free(c);
@@ -596,14 +669,17 @@ static void file_saveas(GtkMenuItem *menuitem, gpointer user_data)
 
 static void file_saveselection(GtkMenuItem *menuitem, gpointer user_data)
 {
+     gint type_id;
+     gboolean use_defs;
      gchar *fn;
      Chunk *c;
      Mainwindow *w = MAINWINDOW(user_data);
-     fn = get_save_filename(NULL,_("Save selection as ..."));
+     fn = get_save_filename(NULL,_("Save selection as ..."),
+			    &type_id,&use_defs);
      if (!fn) return;
      c = chunk_get_part(w->doc->chunk,w->doc->selstart,
 			w->doc->selend-w->doc->selstart);
-     if (!chunk_save(c,fn,dither_editing,w->statusbar))
+     if (!chunk_save(c,fn,type_id,use_defs,dither_editing,w->statusbar))
 	  inifile_set("lastSaveFile",fn);
      gtk_object_sink(GTK_OBJECT(c));     
      g_free(fn);
@@ -1181,7 +1257,7 @@ gtk_window_set_modal(GTK_WINDOW(a),TRUE);
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
      c = gtk_label_new(_("Created by Magnus Hjorth (magnus.hjorth@home.se)\n"
-	  "Copyright 2002-2005, Magnus Hjorth"));
+	  "Copyright 2002-2006, Magnus Hjorth"));
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
      c = gtk_hseparator_new();
@@ -1431,15 +1507,15 @@ static void effects_dialog(GtkMenuItem *menuitem, gpointer user_data)
 static void file_recent(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
-     Chunk *chunk;
+     Document *doc;
      gchar *fn;
      GList *l = w->recent;
      GList *m = recent_filenames;
      while (l->data != menuitem) { l=l->next; m=m->next; }
      fn = g_strdup((gchar *)m->data);
-     chunk = chunk_load(fn, dither_editing, w->statusbar);
-     if (!chunk) { g_free(fn); return; }
-     mainwindow_set_chunk(w, chunk, fn);
+     doc = document_new_with_file(fn, w->statusbar);
+     if (doc == NULL) { g_free(fn); return; }
+     mainwindow_set_document(w, doc, fn);
      inifile_set("lastOpenFile",fn);
      recent_file(fn);
      g_free(fn);
@@ -1958,15 +2034,18 @@ static void mainwindow_value_changed(GtkAdjustment *adjustment,
 static void mainwindow_zoom_changed(GtkAdjustment *adjustment, 
 				    gpointer user_data)
 {
+     gboolean follow_cursor;
      float max_samp,min_samp,target_samp,current_samp;
      Mainwindow *w = MAINWINDOW(user_data);
 
+     /* puts("mainwindow_zoom_changed"); */
      if (setting_zoom_flag) return;
      current_samp = w->doc->viewend - w->doc->viewstart;
      min_samp = GTK_WIDGET(w->view)->allocation.width;
      max_samp = w->doc->chunk->length;
      target_samp = max_samp * pow(min_samp/max_samp,adjustment->value);
-     document_zoom(w->doc,current_samp/target_samp,(target_samp<current_samp));
+     follow_cursor = target_samp <= current_samp;
+     document_zoom(w->doc,current_samp/target_samp,(target_samp<=current_samp));
 }
 
 static void mainwindow_vertical_zoom_changed(GtkAdjustment *adjustment,
@@ -2194,13 +2273,13 @@ GtkWidget *mainwindow_new(void)
 GtkWidget *mainwindow_new_with_file(char *filename)
 {
      Mainwindow *w;
-     Chunk *c;
+     Document *doc;
      gchar *d;
      w = MAINWINDOW ( mainwindow_new() );
      d = make_filename_rooted(filename);
-     c = chunk_load(d, dither_editing, w->statusbar);
-     if (c) {
-          mainwindow_set_chunk(w, c, d);
+     doc = document_new_with_file(d, w->statusbar);
+     if (doc != NULL) {
+          mainwindow_set_document(w, doc, d);
           inifile_set("lastOpenFile",d);
 	  recent_file(d);
      }
